// Dutchtronix AVR Oscilloscope Clock
//
//  Copyright  2010 Johannes P.M. de Rie
//
//  All Rights Reserved
//
//  This file is part of the Dutchtronix Oscilloscope Clock Distribution.
//  Use, modification, and re-distribution is permitted subject to the
//  terms in the file named "LICENSE.TXT", which contains the full text
//  of the legal notices and should always accompany this Distribution.
//
//  This software is provided "AS IS" with NO WARRANTY OF ANY KIND.
//
//  This notice (including the copyright and warranty disclaimer)
//  must be included in all copies or derivations of this software.
//
#include <avr/io.h>
#include <stdlib.h>
#include <string.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>

#include "./config.h"
#include "./util.h"
#include "./ClkISR.h"

uint8 LedCount;
uint8 LedDuration, SavedLedDuration;


extern void InitRTCInt(void);
extern void InitiateSysReset(void);

void setLed(byte v);

#if 0		//NOT USED YET
PushButtonState_t S1;
PushButtonState_t S2;
//
void InitSwitch(void)
{
#if !ONESWITCH
	SWDDR &= ~(_BV(SW1Bit)|_BV(SW2Bit));	//set as input
	SWPort |= (_BV(SW1Bit)|_BV(SW2Bit));	//enable pull-up resistor
#else
	SWDDR &= ~(_BV(SW1Bit));				//set as input
	SWPort |= (_BV(SW1Bit));				//enable pull-up resistor
#endif
}
//
// TC0 is used to program the push buttons
//
// It tracks the duration of a down push and it does key debouncing
//
// Initialize TC0
//
// Set prescaler to clkio/1024. At 20 Mhz, this is 51.2 uSec
// Using a Timer0CountUp value of 127 yields an match every 6.502 mSec
//
void InitTC0(void)
{
	TCCR0A = 0;
	TCCR0B = (1<<CS02)|(0<<CS01)|(1<<CS00);
	OCR0A = Timer0CountUp;
}
//
// Enable TC0 timer for key debouncing
//
void StartKeyDebouncing(void)
{
	GPIOR0 |= _BV(fDebouncing);
	S1.TC0Cntr = 0;
	S2.TC0Cntr = 0;
	S1.KeyState = S2.KeyState = 0xff;
	S1.Position = S2.Position = FALSE;
#if DEBUG
	TC0DebugCnt = 150;
#endif
	TIFR0 = _BV(OCF0A);				//clear pending match
	TIMSK0 = _BV(OCIE0A);			//Enable TC0.matchA
}
//
// Disable TC0 timer for key debouncing
//
void DisableTC0(void)
{
	TIMSK0 = (0<<OCIE0A);			//Disable TC0.matchA
}
//
//
void ProcessButton(PushButtonState_t *pSW, byte ButtonMsk, byte fUPMsk, byte fTimeoutMsk, BOOL fDemoEnabled)
{
#if PORTDSWITCH
	SWPort |= (_BV(SW1Bit));				//re-enable pull-up resistor since CharPixel() will undo it.
#endif
//
// Decrement Timer0Counter iff != 0
//
	if (pSW->TC0Cntr && (--pSW->TC0Cntr == 0))
	{
		//
		// the 8-bit Timer0S1Counter field is now 0
		//
		if (!fDemoEnabled) {								//ignore if in demo mode
			GPIOR0 |= fTimeoutMsk;							//mark timeout in flags
			pSW->Position = FALSE;							//Pretend Button is up (ignore next fUPS1)
			pSW->fAutorepeat = FALSE;
		}
	}
//
// bool_t DebounceSwitch2() Jack Ganssle embedded.com
//    static uint16_t State = 0// // Current debounce status
//    State=(State<<1) | !RawKeyPressed() | 0xe000//
//    if(State==0xf000)return TRUE//
//    return FALSE// 
//
	pSW->KeyState = (uint8_t)(pSW->KeyState << 1);
	if (SWPin & ButtonMsk) {
		pSW->KeyState |= 1;
	}
	if (pSW->KeyState == 0b10000000) {						//7 downs in a row
//		ASSERT(pSW->Position == FALSE);						//gets hit when pushing S1 fast!
		if (pSW->Position == FALSE) {
			pSW->Position = TRUE;
			pSW->TC0Cntr = (TIMER0TIMEOUT & 0xff);
		}
	} else if (pSW->KeyState == 0b01111111) {				//7 ups in a row. Could be Button Up or failed push (<7 downs in a row)
		if (pSW->fAutorepeat) {
			pSW->fAutorepeat  = FALSE;
			GPIOR0 &= ~fTimeoutMsk;
		} else if (pSW->Position != FALSE) {				//If Button already UP (FALSE), ignore this event
			GPIOR0 |= fUPMsk;
		}
		pSW->Position = FALSE;
		pSW->TC0Cntr = 0;
	} else if (pSW->KeyState == 0) {
		//
		// pSW->KeyState==0 indicates key has been down for at least 8 checks.
		// Start autorepeating (if allowed) after S1 has had a timeout (fTimeoutS1)
		//
		if (pSW->okToRepeat && (GPIOR0 & fTimeoutMsk)) {
			if (pSW->fAutorepeat == FALSE) {
				pSW->Position = TRUE;		//Undo "Pretend Button is up" (we want the next fUPS1)
				pSW->autorepeatcnt = AUTOREPEATVAL;
				pSW->fAutorepeat = TRUE;
			} else {
				if (--pSW->autorepeatcnt == 0) {
					//
					// reached 0. Insert an fUPS1 event.
					//
					GPIOR0 |= fUPMsk;	//set UP in flags
					pSW->autorepeatcnt = AUTOREPEATVAL;
				}
			}
		}
	}
}
//
// Timer0 Match Handler. Used to debounce/interpret S1 and S2
//
ISR(TIMER0_COMPA_vect)
{
	TCNT0 = 0;
	sei();											//stay as accurate as possible
#if DEBUG
	if (--TC0DebugCnt == 0) {
		Flags.DBGT = TRUE;				//set DBG Timeout in flags
		TC0DebugCnt = 150;
	}
#endif
	ProcessButton(&S1, _BV(SW1Bit), _BV(fUPS1), _BV(fTimeoutS1), Flags.DemoEnabled);
#if !ONESWITCH
	ProcessButton(&S2, _BV(SW2Bit), _BV(fUPS2), _BV(fTimeoutS2), FALSE);
#endif
//		StopKeyDebouncing();						//stop debouncing if both S1 & S2 done
#if ONESWITCH
	if ((S1.KeyState) == 0xff) 						//all ones means UP. done if both switches are all ones
#else
	if ((S1.KeyState & S2.KeyState) == 0xff) 		//all ones means UP. done if both switches are all ones
#endif
	{
		GPIOR0 &= ~_BV(fDebouncing);
		DisableTC0();								//stop key debouncing timer
	}
}
#endif

volatile uint8 Timer1Counter;
volatile uint8 Timer1Counter2;
//
// TC1 is used for general timeout purposes
//	LED flashing
//	Command Timeout
//	General Purpose Delay
//
// Set prescaler to clkio/1024. At 20 Mhz, this is 51.2 uSec/
// Using a Timer1CountUp value of 1953 yields a match every 100 mSec
//
#if F_CPU==20000000UL
#define Timer1CountUp 1953
#elif F_CPU==8000000UL
#define Timer1CountUp 781
#elif F_CPU==1000000UL
#define Timer1CountUp 97
#else
error: F_CPU unsupported
#endif

void InitTC1(void)
{
	TCCR1A = 0;
	TCCR1B = _BV(WGM12)|(1<<CS02)|(0<<CS01)|(1<<CS00);	// run timer in CTC mode
	OCR1A = Timer1CountUp;
}

void EnableTC1(void)
{
	TIFR1 = (1<<OCF1A);		//clear pending match
	TIMSK1 = (1<<OCIE1A);	//Enable TC0.match
	Timer1Counter = 0;
	Timer1Counter2 = 0;
	LedDuration = SavedLedDuration = LedCount = 0;
}

ISR(TIMER1_COMPA_vect)
{
	if (Timer1Counter != 0) {
		--Timer1Counter;
	}
	if (Timer1Counter2 != 0) {
		--Timer1Counter2;
	}
	if (LedDuration != 0) {
		if (--LedDuration == 0) {
			if (--LedCount) {
#if USE_LED
				sbi(LEDI,LED_BIT);		// more flashes to go, toggle led
#endif
				LedDuration = SavedLedDuration;
			}
		}
	}
	AppTimer1Stuff();
}

//
// TC2 is used to run a 200Hz clock. Normally does NOT run.
//
// Initialize and Enable TC2
// Set prescaler to clkio/1024. At 20 Mhz, this is 51.2 uSec
// Using a Timer2CountUp value of 97 yields an overflow every 4.99 mSec (about 200 Hz)
//
#if F_CPU==20000000UL
#define Timer2CountUp 97
#else
error: F_CPU unsupported
#endif

void EnableTC2(void)
{
	TCNT2 = 0;
	TIFR2 = _BV(OCF2A);							//clear pending match
	TIMSK2 = _BV(OCIE2A);						//Enable TC2.Match A
}

void DisableTC2(void)
{
	TIMSK2 &= ~_BV(OCIE2A);						//Disable TC2.Match A
}

void InitTC2(void)
{
	TCCR2A = _BV(WGM21);			// run timer in CTC mode
	TCCR2B = (1<<CS22)|(1<<CS21)|(1<<CS20);
	OCR2A = Timer2CountUp;
}

ISR(TIMER2_COMPA_vect)
{
}
//
//	Execute a read on the EEPROM
//	arg: eeprom address to read
//	out: data byte
//	no address increment
//
byte EepromRead(uint adr)
{
	do {} while (EECR & _BV(EEPE));
	EEAR = adr;
	EECR |= _BV(EERE);							//read command
	return EEDR;
}
//
// EepromWrite(uint EEpromadr, byte val)
//
void EepromWrite(uint adr, byte val)
{
	do {} while (EECR & _BV(EEPE));
	EEAR = adr;
	EEDR = val;
	byte flags = SREG;
	cli();
	EECR |= _BV(EEMPE);							//write command
	EECR |= _BV(EEPE);
	SREG = flags;
}
//
//UpdateEEprom(uint EEpromadr, byte val)
//
void UpdateEEprom(uint adr, byte val)
{
	EepromWrite(adr, val);
}
//
// Disable Watchdog Timer Completely
//
void WDTimerOff(void)
{
	wdt_reset();
	MCUSR &= ~_BV(WDRF);
	WDTCSR |= (_BV(WDCE) | _BV(WDE));
	WDTCSR = 0;
}
//
// Initiate a System Reset
//
void inline InitiateSysReset(void)
{
	cli();
	WDTCSR |= (_BV(WDCE) | _BV(WDE));
	WDTCSR = (_BV(WDE)|_BV(WDP1));				//Watchdog System Reset Enable, 64 mSec timeout
	while (TRUE);
}

//
// Improved using the amazing elegance of Peter Barrett's code
//
#define MAXTXBUFFERLEN	32
#define MAXRXBUFFERLEN	128
#define XON 0x11
#define XOFF 0x13
#define TXInterruptOn() UCSR0B |= _BV(UDRIE0)
#define TXInterruptOff() UCSR0B &= ~_BV(UDRIE0)
#define SIG_UART_RECV USART0_RX_vect
#define SIG_UART_DATA USART0_UDRE_vect
//#define TX_SUSPENDED	1	No Flow Control on Transmit Channel
#define RX_SUSPENDED	2
#define SEND_XOFF		4
#define SEND_XON		8
#define OVERRUN			16
#define UNDERRUN		32

#define XOFFLEVEL (3*MAXRXBUFFERLEN/4)
#define XONLEVEL (MAXRXBUFFERLEN/4)

char TXBuffer[MAXTXBUFFERLEN];
volatile byte TXBufferHead;
volatile byte TXBufferTail;
char RXBuffer[MAXRXBUFFERLEN];
volatile byte RXBufferHead;
volatile byte RXBufferTail;
extern uint8 GlobalFault;
uint8 UARTFlags;

uint32 BaudTable[11] PROGMEM = {
	19200,
	28800,
	38400,
	57600,
	115200,
	128000,
	230400,
	250000,
	4800,
	9600,
	14400
};

uint8 UARTInit(byte newBaud)
{
	UARTFlags = 0;
	UCSR0A = 0;
	if (0 == UARTSetBaudVal(newBaud)) {
		return 0;
	}
// set/enable rx interrupt, set tx interrupt, clear data empty interrupt
	UCSR0B |= _BV(RXEN0)|_BV(TXEN0)|_BV(RXCIE0);
// set default 8 bits, no parity, 1 stop
	UCSR0C = (_BV(UCSZ01)|_BV(UCSZ00));
#if DEBUG
//
// all memory cleared in RELEASE mode
//
	RXBufferHead = 0;
	RXBufferTail = 0;
	TXBufferHead = 0;
	TXBufferTail = 0;
#endif
	return 1;
}
//
// Input: internal baud rate code
//
// Map internal baud rate code to baud rate value for AVR
// and set in UART
//
uint8 UARTSetBaudVal(byte newBaud)
{
	//Overkill.
	byte sreg = SREG;
	cli();
	byte idx;
	if (newBaud >= BAUD_MAX) {
		return 0;
	}
	idx = newBaud - BAUD_BASE;			//work around what looks like a bug.
	uint32 baud = pgm_read_dword(&BaudTable[idx]);
	uint16 baud_setting = (F_CPU / 4 / baud - 1) >> 1;
	UCSR0A |= _BV(U2X0);
	UBRR0 = baud_setting;
	SREG = sreg;
	return 1;
}

void UARTStop(void)
{
	UCSR0A = 0;
	UCSR0B = 0;
	UCSR0C = 0;
	RXBufferHead = 0;
	RXBufferTail = 0;
	TXBufferHead = 0;
	TXBufferTail = 0;
}

SIGNAL(SIG_UART_DATA)
{
	if (UARTFlags & SEND_XON) {
		UDR0 = XON;
		UARTFlags &= ~SEND_XON;
		return;
	}

	if (UARTFlags & SEND_XOFF) {
		UDR0 = XOFF;
		UARTFlags &= ~SEND_XOFF;
		return;
	}

	if ((TXBufferHead - TXBufferTail) != 0) {
		UDR0 = TXBuffer[TXBufferTail++ & (MAXTXBUFFERLEN-1)];
	} else {
		TXInterruptOff();					// Stop tx interrupt for now
	}
}

//	Timeout etc
uint8 Serial_Wait()
{
	return 1;
}

//
// void UARTSendByte(char ch, BOOL fForce)
//
// Assumes interrupts are ON. Do not call from an ISR.
//
uint8 UARTSendByte(byte b, BOOL fWait)
{

	while ((TXBufferHead - TXBufferTail) == MAXTXBUFFERLEN) {
		if (!Serial_Wait() || !fWait) {
			return 0;
		}
	}
	TXBuffer[TXBufferHead++ & (MAXTXBUFFERLEN-1)] = b;
	TXInterruptOn();
	return 1;
}
//
//
uint8 UARTRXAvailable(void)
{
	return (RXBufferHead - RXBufferTail);
}

void Serial_In (uint8 b)
{
	uint8 a = RXBufferHead - RXBufferTail;
	if ((a >= XOFFLEVEL) && !(UARTFlags & RX_SUSPENDED)) {
		UARTFlags |= (RX_SUSPENDED | SEND_XOFF);
		TXInterruptOn();	// Start Sending
		LED2_ON;
	}

	if (a < MAXRXBUFFERLEN) {
		RXBuffer[RXBufferHead++ & (MAXRXBUFFERLEN-1)] = b;
//		GPIOR0 |= _BV(fRcvdChar);
	}
	else {
		UARTFlags |= OVERRUN;
		GlobalFault = LCDOVERFLOW;
	}
}
//
// UART Receive Complete Interrupt
//
ISR(SIG_UART_RECV)
{
	uint8 err = UCSR0A & (_BV(DOR0)|_BV(FE0));
	uint8 b = UDR0;
	if (err) {
		return;	// overrun or framing error
	}
	//
	// Cannot support XON/XOFF on the Transmit Channel since the receiver
	// would send XON/XOFF bytes on this Receive Channel, mixing those with
	// the binary data from the AVCX protocol.
	//
	Serial_In(b);
}
//
// RXBufferHead may change while interrupts are on
//
uint8 UARTReceiveByte(void)
{
	uint8 a;
	byte flags;
	ASSERT(UARTRXAvailable() != 0);
	while (!(a = (RXBufferHead - RXBufferTail))) {
		if (!Serial_Wait()) {
			return 0;	//no good
		}
	}
	//	XON if required
 	flags = SREG;
	cli();
	if ((UARTFlags & RX_SUSPENDED) && (a < XONLEVEL)) {
		UARTFlags |= SEND_XON;
		UARTFlags &= ~RX_SUSPENDED;
		TXInterruptOn();	// start sending
		LED2_OFF;
	}
	SREG = flags;
	return RXBuffer[RXBufferTail++ & (MAXRXBUFFERLEN-1)];
}
//
// Print String
//
void UARTPrintfStr(char *pBuf)
{
	char ch;
	while ((ch = *pBuf++) != 0) {
		UARTSendByte(ch, TRUE);
	}
}
//
// Print flash based text
//
void UARTPrintfProgStr(PGM_P pBuf)
{
	char ch;
	while ((ch = pgm_read_byte(pBuf)) != 0) {
		UARTSendByte(ch, TRUE);
		++pBuf;
	}
}
//
//
//
void UARTPrintfint(int d)
{
#if 0
    if (d < 0)
    {
        UARTSendByte('-', FALSE);
        d = -d;
    }
    
    uint8 buf[16];
    uint8 i = 0;
    
    while (d > 0) {
        buf[i++] = '0' + (d % 10);
        d /= 10;
    }
    while (i) {
        UARTSendByte(buf[--i], FALSE);
	}
#else
    char buf[16];
	itoa(d, buf, 10);
	UARTPrintfStr(buf);
#endif
}
//
//
#if 0
BOOL UARTReceiveLine(char *pDst, byte maxLength)
{
	char ch;
	byte head, tail, l = 0;
	byte sreg = SREG;
	cli();
	ASSERT(GPIOR0 & _BV(fRcvdChar));
	head = RXBufferHead;
	tail = RXBufferTail;
	do {
		ch = RXBuffer[tail];
		tail = (tail + 1) & RXBUFFERMASK;
		if (++l <= maxLength) {
			*pDst++ = ch;
		}
	} while ((ch != LF) && (head != tail));
//
// Change last character to NULL. It's an LF or the line was incomplete.
//
	*--pDst = 0;						//TODO fails if maxLength == 0
	if (head == tail) {					//is buffer now empty?
		GPIOR0 &= ~_BV(fRcvdChar);		//buffer now empty
	}
	RXBufferTail = tail;
	SREG = sreg;
	return l;
}
//
// Flush remainder of current record (until LF or end)
//
void UARTFlushRecord(void)
{
	char ch;
	byte head, tail;
	if (GPIOR0 & _BV(fRcvdChar)) {
		byte sreg = SREG;
		cli();
		head = RXBufferHead;
		tail = RXBufferTail;
		do {
			ch = RXBuffer[tail];
			tail = (tail + 1) & RXBUFFERMASK;
		} while ((ch != LF) && (head != tail));
		if (head == tail) {						//is buffer now empty?
			GPIOR0 &= ~_BV(fRcvdChar);			//yes
		}
		RXBufferTail = tail;
		SREG = sreg;
	}
}
#endif
//
//
void UARTKillPendingOutput(void)
{
	TXBufferHead = 0;
	TXBufferTail = 0;
	//wait for current send, if any, to complete
	while (UCSR0B & _BV(UDRIE0)) ;	//wait for TXInterrupt to finish
}


//
// Immediate version. Should work with interrupts off too,
// provided UART_SendByte() has not been called in isolation.
// UNTESTED
//
void UART_PrintfProgStr_I(prog_char* pBuf)
{
	uint8 ch;
	while (UCSR0B & _BV(UDRIE0)) ;	//wait for TXInterrupt to finish
	while ((ch = pgm_read_byte(pBuf)) != 0) {
		UDR0 = ch;
		++pBuf;
		while ((UCSR0A & _BV(TXC0)) == 0) ;
		UCSR0A |= _BV(TXC0);
	}
}
