/*
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software Foundation, 
  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
//
// LCD controller for Epson P300013100 display.
// See LCD_EpsonP3000.c for description.
//
// Can reduce refresh time by unrolling SED1180 loop completely:
// 20 * 4 cycles = 4 uSec.
//
#include "./avr.inc"
#include <avr/io.h>
#include <avr/portpins.h>

#include "Commoncfg.h"

	.extern TextPage
	.extern Font8x8

//-- map the IO register back into the IO address space
#define PA_DDR		(_SFR_IO_ADDR(PORTA) - 1)
#define PB_DDR		(_SFR_IO_ADDR(PORTB) - 1)
#define PC_DDR		(_SFR_IO_ADDR(PORTC) - 1)
#define PD_DDR		(_SFR_IO_ADDR(PORTD) - 1)
#define PA_OUT		_SFR_IO_ADDR(PORTA)
#define PB_OUT		_SFR_IO_ADDR(PORTB)
#define PC_OUT		_SFR_IO_ADDR(PORTC)
#define PD_OUT		_SFR_IO_ADDR(PORTD)
#define PA_IN		(_SFR_IO_ADDR(PORTA) - 2)
#define PB_IN		(_SFR_IO_ADDR(PORTB) - 2)
#define PC_IN		(_SFR_IO_ADDR(PORTC) - 2)
#define PD_IN		(_SFR_IO_ADDR(PORTD) - 2)
//
// GENERAL DEFINITIONS
//
// Vx is a memory nibble address, not a pixel address
// Vy is line number (pixel and memory values are identical)
//
// PB0..PD3 connected to D0..D3 (nibble data)
//
#define	SRAMDATAI	PB_IN
#define	SRAMDATAO	PB_OUT
#define	SRAMDATA_DDR PB_DDR
//
// PD2 connected to YSCL
//
#define	LCD_YSCL	PD_OUT,YSCL_BIT
#define	YSCL_DDR	PD_DDR,YSCL_BIT
//
// PD3 connected to M (AC) Line
//
#define	M_DDR		PD_DDR,3
#define	LCD_MTOGGLE	PD_IN,M_BIT
//
// PD4 connected to LCD Enable
//
#define	LCD_ENABLE	PD_OUT,ENABLE_BIT
#define	ENABLE_DDR	PD_DDR,ENABLE_BIT
//
// PD5 connected to LCD HSYNC (LP)
//
#define	LCD_HSYNC	PD_OUT,HSYNC_BIT
#define	HSYNC_DDR	PD_DDR,HSYNC_BIT
//
// PD6 connected to LCD VSYNC (SF)
//
#define	LCD_VSYNC	PD_OUT,VSYNC_BIT
#define	VSYNC_DDR	PD_DDR,VSYNC_BIT
//
// PD7 connected to software DOTCLOCK (XSCL)
//
#define	DOTCLK		PD_OUT,DOTCLK_BIT
#define	DOTCLK_PORT	PD_OUT
#define	DOTCLK_DDR	PD_DDR,DOTCLK_BIT
//
// PC0 connected to MCU_TEST bit
//
#define MCU_TEST		PC_OUT,MCU_TEST_BIT
#define MCU_TEST_DDR	DDRC,MCU_TEST_BIT
//
// PC1 connected to LCD ECL
//
#define	LCD_ECL		PC_OUT,ECL_BIT
#define	ECLDDR		PC_DDR,ECL_BIT
//
// PA0 connected to LED
//
#define	LEDI		PA_IN,0
#define	LEDO		PA_OUT,0
#define	LED_DDR		PA_DDR,0
//
// PA1 connected to LED2
//
#define	LED2I		PA_IN,1
#define	LED2O		PA_OUT,1
#define	LED2_DDR	PA_DDR,1

#define COLORWHITE	0b00000001
#define COLORBLACK	0b00000000

#define BITSPERPIXEL	1
#define DATABITS		4

.section	.text

#define	PORTDCLKOFF		r21
#define	PORTDCLKON		r20

        .global TIMER0_COMPA_vect
//
// Assume non-data pins in PORTB are input
//
	.macro	UpdateColumns
	ld		r16,Z+
	swap	r16				;ldata >> 4
	out		DOTCLK_PORT,PORTDCLKON			; sbi	DOTCLK
	out		SRAMDATAO,r16
	out		DOTCLK_PORT,PORTDCLKOFF			; cbi	DOTCLK
	swap	r16				;ldata << 4
	out		DOTCLK_PORT,PORTDCLKON			; sbi	DOTCLK
	out		SRAMDATAO,r16
	out		DOTCLK_PORT,PORTDCLKOFF			; cbi	DOTCLK
	.endm
//
// TC0MatchHandler
//
// Executed every 198.4 uSec (5,000 Hz).
//
// Refreshes one display line.
// 100 lines -> 50 Screen refreshes / second
//
TIMER0_COMPA_vect:
	push	r16
	in		r16, _SFR_IO_ADDR(SREG)
	push	r16
#if 0
Not needed when running in CTC mode
	clr		r16
	sts		TCNT0,r16
#endif
	push	ZL
	push	ZH
	push	r18
	push	r19
	push	PORTDCLKOFF
	push	PORTDCLKON
	push	r0
	push	r1
#if DEBUG
	sbi		MCU_TEST
#endif
//
// get the line we're refreshing. Compute data ptr.
// todo: times 160 => times 128 + times 32
//
	lds		r19,RefreshLine			// Y
	ldi		r16,160					//160 bytes per line
	mul		r19,r16
	movw	ZL,r0
	eor		r1,r1
	subi	ZL,lo8(-(FrameBuffer))
	sbci	ZH,hi8(-(FrameBuffer))
//
//	Allow interrupts again (process Serial In while doing refresh)
//
	sei
//
	ldi		r18,20						//1180 count
#if 1
	tst		r19							//RefreshLine
	brne	1f
	sbi		LCD_VSYNC					//start VSYNC 
1:
#endif
//
// preload the clock toggle registers
//
	in		PORTDCLKOFF,DOTCLK_PORT
	mov		PORTDCLKON,PORTDCLKOFF
	sbr		PORTDCLKON,_BV(DOTCLK_BIT)	//DotClk On
SED1180Loop:
	UpdateColumns
	UpdateColumns
	UpdateColumns
	UpdateColumns
	UpdateColumns
	UpdateColumns
	UpdateColumns
	UpdateColumns
//
// Transfer the 1180 to the next 1180
//
	sbi	LCD_ECL
	cbi	LCD_ECL	
//
// Loop until all 20 1180s had their turn
//
	dec		r18
	breq	1f
	rjmp	SED1180Loop
1:
//
// Horizontal Sync.
//
	sbi		LCD_ECL				//needed!!
	sbi		LCD_YSCL
	sbi		LCD_HSYNC
	cbi		LCD_ECL				//PORTC!
#if 0
	tst		r19					//RefreshLine
	brne	1f
	sbi		LCD_VSYNC			//start VSYNC 
1:
#endif
	cbi		LCD_YSCL			//exact sequence is essential
	cbi		LCD_HSYNC
	cbi		LCD_VSYNC			//stop VSYNC

//
// Update Line Address
//
	inc		r19
	cpi		r19,PHYS_LINES
;	brsh	DoLCD_LastLine
;	rjmp	RefreshExit			//done
	brlo	RefreshExit

DoLCD_LastLine:
//
//toggle AC line
//
	sbi		LCD_MTOGGLE
//
// start new frame
//
	clr		r19
//
// Enable display after 3 frames
//
	lds		r16,pwrup_state
	cpi		r16,3
	brsh	Turn_On_LCDX
	inc 	r16
	sts		pwrup_state,r16
	rjmp	RefreshExit
Turn_On_LCDX:
	sbi		LCD_ENABLE
RefreshExit:
	sts		RefreshLine,r19
#if DEBUG
	cbi		MCU_TEST
#endif
	pop		r1
	pop		r0
	pop		PORTDCLKON
	pop		PORTDCLKOFF
	pop		r19
	pop		r18
	pop		ZH
	pop		ZL
Tdone:
	pop		r16
	out		_SFR_IO_ADDR(SREG),r16
	pop		r16
	reti

#undef	PORTDCLKOFF
#undef	PORTDCLKON

	.global	InitMem

#define	RAMSTART 0x0100

InitMem:
	push	r18
	push	r20
	push	r21
	push	ZL
	push	ZH
	in		r20,_SFR_IO_ADDR(SPL)					;top
	in		r21,_SFR_IO_ADDR(SPH)
	
	ldi		r18,0xcc
;	ldiw	Z,RAMSTART			;Preset RAM
	ldiw	Z,__heap_start
1:
	st		Z+,r18
	cp		ZL,r20
	cpc		ZH,r21
	brne	1b
	pop		ZH
	pop		ZL
	pop		r21
	pop		r20
	pop		r18
	ret


//##########################################################
//##                     ###################################
//## FONTS & BITMAPS     ###################################
//##                     ###################################

//##########################################################
	.global	Font8x8
	.global	Font8x8End

#include ".\Font8x8.inc"

Font8x8End:
	.align

#if 0
	.global	DialData
	.global DialDigits
	.global SecPtrData
	.global MinPtrData
	.global HrPtrData

#include "./EpsonClk.inc"
#endif
	.align

#if SPLASHSCREEN
#include ".\RoseBM.inc"

	.global	SplashScreen
#endif

	.align
