//
// LCD controller sample application by Jan de Rie
// Last update: January 9, 2011
//
#include <conio.h>
#include <stdio.h>
#include <time.h>
#include <windows.h>
#include <string.h>
#include "bitmap.h"

#define LCD_DMF50840 0
#define LCD_EPSONP300013100 1

#define GENBITMAP 0

#define SOFT_FLOWCONTROL 0

#define LCDDEMO 0xB8
#define LCDCLRSCRN 0x73
#define LCDLIGHTON 0x78
#define LCDLIGHTOFF 079
#define LCD_DUMMY 0x7D
#define LCDCLOCKON 0x7E
#define LCDCLOCKOFF 0x7F
#define	LCDCOLOR 0x88
#define LCDPSET 0x44
#define LCDPUNSET 0x50
#define LCDXPLUS 0x41
#define LCDYPLUS 0x42
#define LCDXMINUS 0x49
#define LCDYMINUS 0x4a
#define LCDPSETXPLUS 0x45
#define LCDPSETYPLUS 0x46
#define LCDPSETXMINUS 0x4d
#define LCDPSETYMINUS 0x4e
#define LCDPUNSETXPLUS 0x55
#define LCDPUNSETYPLUS 0x56
#define LCDPUNSETXMINUS 0x59
#define LCDPUNSETYMINUS 0x5e
#define LCDPRNTSTRING 0xb0
#define LCDPUTCHAR 0xa0
#define	LCDSETSCREEN 0x98
#define LCDDRAWLINE 0xa8	// takes 4 arguments (8 bytes)
#define LCDDRAWCIRCLE 0xc8	// takes 3 arguments (6 bytes)
#define LCDSETY 0x90		// takes 2 byte arguments
#define LCDSETX 0x91		// takes 2 bytes argument
#define LCDSETYCLR 0x92		// SetY, Clr Pixel
#define LCDSETXCLR 0x93		// SetX, Clr Pixel
#define LCDSETYSET 0x94		// SetY, Set Pixel
#define LCDSETXSET 0x95		// SetX, Set Pixel
#define	LCDBITMAP 0xc0
#define LCDSERIALTEST 0xd0

// Flow control flags

#define FC_DTRDSR 0x01
#define FC_RTSCTS 0x02
#define FC_XONXOFF 0x04

#if LCD_DMF50840
#define AVCXPIXELS 320
#define AVCXLINES 240
#define CLOCKOFFSET 0
#elif LCD_EPSONP300013100
#define AVCXPIXELS 640
#define AVCXLINES 200
#define CLOCKOFFSET 30
#endif
#define LastLine (AVCXLINES - 1)
#define CHAR_HEIGHT 8
#define CHAR_WIDTH 8

// ascii definitions

#define ASCII_BEL 0x07
#define ASCII_BS 0x08
#define ASCII_LF 0x0A
#define ASCII_CR 0x0D
#define ASCII_XON 0x11
#define ASCII_XOFF 0x13


typedef          char s08;
typedef          short s16;
typedef unsigned char u08;
typedef unsigned short u16;
typedef unsigned int  u32;

// variables used with the com port
BOOL bPortReady;
DCB dcb;
COMMPROP prop;
COMMTIMEOUTS CommTimeouts;
BOOL bWriteRC;
BOOL bReadRC;
DWORD iBytesWritten;
DWORD dwRead, dwErr;
HANDLE SerPrt = INVALID_HANDLE_VALUE;
OVERLAPPED osReader = {0};
DWORD dwEvtMask;
BOOL fWaitingOnRead = FALSE;
#if SOFT_FLOWCONTROL
BOOL fStopTransmitting = FALSE;
#endif
#define READ_BUF_SIZE 1
char InBuf[READ_BUF_SIZE];

HANDLE SerialInit(char *ComPortName, int BaudRate) 
{
	HANDLE hCom;
	BOOL result;

	hCom = CreateFileA(ComPortName, 
			GENERIC_READ | GENERIC_WRITE,
			0, // exclusive access
			NULL, // no security
			OPEN_EXISTING,
			FILE_FLAG_OVERLAPPED, // overlapped I/O
			NULL); // null template

	if (hCom == INVALID_HANDLE_VALUE) {
		DWORD err = GetLastError();
		char lpMsg[512];
		FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
			  FORMAT_MESSAGE_FROM_SYSTEM |
			  FORMAT_MESSAGE_IGNORE_INSERTS,
			  NULL, GetLastError(),
			  MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
			  (LPTSTR)&lpMsg, 0, NULL);
		return hCom;
	}
	
	result = GetCommProperties(hCom, &prop);
	if (0 == result) {
		printf("SerialInit: Unable to get Serial Device Properties\n");
		CloseHandle(hCom);
		return INVALID_HANDLE_VALUE;
	}
	bPortReady = SetupComm(hCom, 2, 128); // set buffer sizes
	if (0 == bPortReady) {
		dwErr = GetLastError();
		printf("SerialInit: Unable to SetupComm Serial Port. Error = %d\n", dwErr);
		CloseHandle(hCom);
		return INVALID_HANDLE_VALUE;
	}


	bPortReady = GetCommState(hCom, &dcb);
	if (0 == bPortReady) {
		dwErr = GetLastError();
		printf("SerialInit: Unable to GetCommState Serial Port. Error = %d\n", dwErr);
		CloseHandle(hCom);
		return INVALID_HANDLE_VALUE;
	}

	dcb.BaudRate = BaudRate;
	dcb.ByteSize = 8;
	dcb.Parity = NOPARITY;		// EVENPARITY;
	dcb.StopBits = ONESTOPBIT;
	dcb.fAbortOnError = TRUE;

	// set XON/XOFF
#if SOFT_FLOWCONTROL
	dcb.fOutX = FALSE; // XON/XOFF off for transmit
#else
	dcb.fOutX = TRUE; // XON/XOFF off for transmit
#endif
	dcb.fInX = FALSE; // XON/XOFF off for receive
	// set RTSCTS
	dcb.fOutxCtsFlow = FALSE; // turn on CTS flow control
	dcb.fRtsControl = RTS_CONTROL_DISABLE; // 
	// set DSRDTR
	dcb.fOutxDsrFlow = FALSE; // turn on DSR flow control
	dcb.fDtrControl = DTR_CONTROL_DISABLE; // DTR_CONTROL_HANDSHAKE;

	bPortReady = SetCommState(hCom, &dcb);

	if (0 == bPortReady) {
		dwErr = GetLastError();
		printf("SerialInit: Unable to SetCommState Serial Port. Error = %d\n", dwErr);
		CloseHandle(hCom);
		return INVALID_HANDLE_VALUE;
	}

	// Communication timeouts are optional

	bPortReady = GetCommTimeouts (hCom, &CommTimeouts);
	if (0 == bPortReady) {
		dwErr = GetLastError();
		printf("SerialInit: Unable to GetCommTimeouts Serial Port. Error = %d\n", dwErr);
		CloseHandle(hCom);
		return INVALID_HANDLE_VALUE;
	}

	CommTimeouts.ReadIntervalTimeout = 2500;
	CommTimeouts.ReadTotalTimeoutConstant = 2500;
	CommTimeouts.ReadTotalTimeoutMultiplier = 1000;
	CommTimeouts.WriteTotalTimeoutConstant = 2500;
	CommTimeouts.WriteTotalTimeoutMultiplier = 1000;

	bPortReady = SetCommTimeouts (hCom, &CommTimeouts);
	if (0 == bPortReady) {
		dwErr = GetLastError();
		printf("SerialInit: Unable to SetCommTimeouts Serial Port. Error = %d\n", dwErr);
		CloseHandle(hCom);
		return INVALID_HANDLE_VALUE;
	}

	// Create Read event

	osReader.hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);

	if (NULL == osReader.hEvent) {
		printf("SerialInit: failure to create reader hEvent.\n");
		CloseHandle(hCom);
		return INVALID_HANDLE_VALUE;
	}

	return hCom;
}

void ProcessReadResult(void)
{
	int i;
	for (i = 0; i < (int)dwRead; i++) {
#if SOFT_FLOWCONTROL
		if (InBuf[i] == ASCII_XOFF) {
			fStopTransmitting = TRUE;
			printf(" XOFF ");
		} else if (InBuf[i] == ASCII_XON) {
			fStopTransmitting = FALSE;
			printf(" XON ");
		} else
#endif
		{
			printf("%c", InBuf[i]);
		}
	}
	fflush(stdout);
}

void SerialStartRead(void)
{
	int dwErr;
	if (!fWaitingOnRead) {
		// Issue Read Operation
		if (!ReadFile(SerPrt, InBuf, READ_BUF_SIZE, &dwRead, &osReader)) {
			dwErr = GetLastError();
			if ((ERROR_IO_PENDING != dwErr) && (ERROR_IO_INCOMPLETE != dwErr)) {
				printf("SerialStartRead: ReadFile failed error: %d ", dwErr);
				ClearCommError(SerPrt, &dwErr, NULL);
				printf("detail: %x\n", dwErr);
			} else {
				fWaitingOnRead = TRUE;
			}
		} else {
			//ReadFile finished
			ProcessReadResult();
		}
	}
}

void SerialCheckRead(void)
{
	DWORD dwRes;
	int dwErr;

	if (fWaitingOnRead) {
		dwRes = WaitForSingleObject(osReader.hEvent, 0);	// don't wait
		switch (dwRes) {
		case WAIT_OBJECT_0:	// Read Completed
			if (!GetOverlappedResult(SerPrt, &osReader, &dwRead, FALSE)) {
				dwErr = GetLastError();
				if ((ERROR_IO_PENDING != dwErr) && (ERROR_IO_INCOMPLETE != dwErr)) {
					printf("SerialCheckRead: GetOverlappedResult failed error: %d ", dwErr);
					ClearCommError(SerPrt, &dwErr, NULL);
					printf("detail: %x\n", dwErr);
				} else {
					// Still waiting
				}
				break;
			}
			// success
			ProcessReadResult();
			fWaitingOnRead = FALSE;		// ready for another operation
			break;
		case WAIT_TIMEOUT:
			// still waiting
			break;
		default:
			// error
			printf("SerialCheckRead: communication error: %d\n", dwRes);
		}
	}
}

void MySleep(int mSec)
{
	int i;
	if (mSec >= 100) {
		i = mSec / 100;
		for (i = 0; i < (mSec/100); i++) {
			SerialStartRead();
			SerialCheckRead();
			Sleep(100);
		}
	} else {
		SerialStartRead();
		SerialCheckRead();
		Sleep(mSec);
	}
}


void SerialPutc(HANDLE *hCom, unsigned char txchar)
{
	BOOL bWriteRC;
	static DWORD iBytesWritten;
	OVERLAPPED osWrite = {0};
	DWORD dwRes;

#if SOFT_FLOWCONTROL
	if (fStopTransmitting) {
//		printf(" Blocked ");
		while (fStopTransmitting) {
			MySleep(1);
		}
//		printf(" Unblocked ");
	}
#endif

	// Create an event object for write operation

    osWrite.hEvent = CreateEvent(
        NULL,   // default security attributes 
        TRUE,   // manual-reset event 
        FALSE,  // not signaled 
        NULL    // no name
		);
    
	if (NULL == osWrite.hEvent) {
		printf("Error: unable to create Write Operation Event\n");
		return;
	}

	bWriteRC = WriteFile(*hCom, &txchar, 1, &iBytesWritten,&osWrite);
	if (!bWriteRC) {
		// Check if Write is pending. 
		if (ERROR_IO_PENDING != GetLastError()) {
			printf("SerialPutc: unexpected return %d from WriteFile()\n", GetLastError());
			exit(1);
			return;
		}
		dwRes = WaitForSingleObject(osWrite.hEvent, INFINITE);
		switch (dwRes) {
		case WAIT_OBJECT_0:
			if (0 == GetOverlappedResult(*hCom, &osWrite, &iBytesWritten, TRUE)) {
				printf("GetOverlappedResult failed with error %d.\n", GetLastError());
			} else {
				// success
			}
			break;
		default:
			printf("WaitForSingleObject failed with error %d.\n", dwRes);
			break;
		}
	} else {
		// WriteFile() returned TRUE
		DWORD err = GetLastError();	//returns 995, Operation Aborted
	}
	CloseHandle(osWrite.hEvent);

	SerialStartRead();
	SerialCheckRead();
}

void AVCXClearScreen(void)
{
	SerialPutc(&SerPrt, LCDCLRSCRN);
}

void AVCXSetScreen(int color, int x0, int y0, int x1, int y1 )
{
	SerialPutc(&SerPrt, LCDSETSCREEN);
	SerialPutc(&SerPrt, color);
	SerialPutc(&SerPrt, (x0 & 0xff));
	SerialPutc(&SerPrt, (x0 >> 8));
	SerialPutc(&SerPrt, (y0 & 0xff));
	SerialPutc(&SerPrt, (y0 >> 8));
	SerialPutc(&SerPrt, (x1 & 0xff));
	SerialPutc(&SerPrt, (x1 >> 8));
	SerialPutc(&SerPrt, (y1 & 0xff));
	SerialPutc(&SerPrt, (y1 >> 8));
	MySleep(500);
}

void AVCXDemo(int val)
{
	SerialPutc(&SerPrt, LCDDEMO);
	SerialPutc(&SerPrt, (unsigned char)val);
}

void AVCXSetRGBColor(int Blue, int Green, int Red)
{
	//SerialPutc(&SerPrt, LCDCOLOR | (Red << 2) | (Green << 1) | Blue);
}

void AVCXSetBWColor(int color)
{
	SerialPutc(&SerPrt, LCDCOLOR | color);
	SerialPutc(&SerPrt, 0);
}

void AVCXClockOn(void)
{
	SerialPutc(&SerPrt, LCDCLOCKON);
}

void AVCXPSet(int opcode)
{
	SerialPutc(&SerPrt, opcode);
}

void AVCXClockOff(void)
{
	SerialPutc(&SerPrt, LCDCLOCKOFF);
	SerialPutc(&SerPrt, LCDCLRSCRN);
}

void AVCXPrintString(char * txt)
{
	char *txt1 = txt;
	SerialPutc(&SerPrt, LCDPRNTSTRING);
	while (*txt) {
		SerialPutc(&SerPrt,*txt++);
	}
	SerialPutc(&SerPrt,0);
}

void AVCXPutChar(char ch)
{
	SerialPutc(&SerPrt, LCDPUTCHAR);
	SerialPutc(&SerPrt, ch);
}

void AVCXSetXY(int x, int y)
{
	SerialPutc(&SerPrt, LCDSETX);
	SerialPutc(&SerPrt, (x & 0xff));
	SerialPutc(&SerPrt, (x >> 8));
	SerialPutc(&SerPrt, LCDSETY);
	SerialPutc(&SerPrt, (y & 0xff));
	SerialPutc(&SerPrt, (y >> 8));
}

void AVCXSetXYPSet(unsigned int x, unsigned int y)
{
	SerialPutc(&SerPrt, LCDSETX);
	SerialPutc(&SerPrt, (x & 0xff));
	SerialPutc(&SerPrt, (x >> 8));
	SerialPutc(&SerPrt, LCDSETYSET);
	SerialPutc(&SerPrt, (y & 0xff));
	SerialPutc(&SerPrt, (y >> 8));
}

void AVCXSetXYPClr(unsigned int x, unsigned int y)
{
	SerialPutc(&SerPrt, LCDSETX);
	SerialPutc(&SerPrt, (x & 0xff));
	SerialPutc(&SerPrt, (x >> 8));
	SerialPutc(&SerPrt, LCDSETYCLR);
	SerialPutc(&SerPrt, (y & 0xff));
	SerialPutc(&SerPrt, (y >> 8));
}
int AVCXHalfPage(char *str, int Xpos)
{
	int Ypos;
	Ypos = 0;
	AVCXSetRGBColor(1,0,0);
	AVCXSetXY(Xpos, Ypos);
	AVCXPrintString(str); Ypos += CHAR_HEIGHT; Xpos += CHAR_WIDTH;
	AVCXSetXY(Xpos, Ypos);
	AVCXSetRGBColor(0,1,0);
	AVCXPrintString(str); Ypos += CHAR_HEIGHT; Xpos += CHAR_WIDTH;
	AVCXSetXY(Xpos, Ypos);
	AVCXSetRGBColor(0,0,1);
	AVCXPrintString(str); Ypos += CHAR_HEIGHT; Xpos += CHAR_WIDTH;
	AVCXSetXY(Xpos, Ypos);
	AVCXSetRGBColor(0,1,0);
	AVCXPrintString(str); Ypos += CHAR_HEIGHT; Xpos += CHAR_WIDTH;
	AVCXSetXY(Xpos, Ypos);
	AVCXSetRGBColor(1,0,0);
	AVCXPrintString(str); Ypos += CHAR_HEIGHT; Xpos += CHAR_WIDTH;
	AVCXSetXY(Xpos, Ypos);
	AVCXSetRGBColor(0,0,0);
	AVCXPrintString(str); Ypos += CHAR_HEIGHT; Xpos += CHAR_WIDTH;
	AVCXSetXY(Xpos, Ypos);
	AVCXSetRGBColor(1,0,0);
	AVCXPrintString(str); Ypos += CHAR_HEIGHT; Xpos += CHAR_WIDTH;
	AVCXSetXY(Xpos, Ypos);
	AVCXSetRGBColor(0,1,0);
	AVCXPrintString(str); Ypos += CHAR_HEIGHT; Xpos -= CHAR_WIDTH;
	AVCXSetXY(Xpos, Ypos);
	AVCXSetRGBColor(0,0,1);
	AVCXPrintString(str); Ypos += CHAR_HEIGHT; Xpos -= CHAR_WIDTH;
	AVCXSetXY(Xpos, Ypos);
	AVCXSetRGBColor(0,1,0);
	AVCXPrintString(str); Ypos += CHAR_HEIGHT; Xpos -= CHAR_WIDTH;
	AVCXSetXY(Xpos, Ypos);
	AVCXSetRGBColor(1,0,0);
	AVCXPrintString(str); Ypos += CHAR_HEIGHT; Xpos -= CHAR_WIDTH;
	AVCXSetXY(Xpos, Ypos);
	AVCXSetRGBColor(0,0,0);
	AVCXPrintString(str); Ypos += CHAR_HEIGHT; Xpos -= CHAR_WIDTH;
	AVCXSetXY(Xpos, Ypos);
	AVCXSetRGBColor(0,1,1);
	AVCXPrintString(str); Ypos += CHAR_HEIGHT; Xpos += CHAR_WIDTH;
	AVCXSetXY(Xpos, Ypos);
	AVCXSetRGBColor(1,0,1);
	AVCXPrintString(str); Ypos += CHAR_HEIGHT; Xpos += CHAR_WIDTH;
	AVCXSetXY(Xpos, Ypos);
	AVCXSetRGBColor(1,1,0);
	AVCXPrintString(str); Ypos += CHAR_HEIGHT; Xpos += CHAR_WIDTH;
	AVCXSetXY(Xpos, Ypos);
	AVCXSetRGBColor(0,1,0);
	AVCXPrintString(str); Ypos += CHAR_HEIGHT; Xpos += CHAR_WIDTH;
	AVCXSetXY(Xpos, Ypos);
	AVCXSetRGBColor(1,0,0);
	AVCXPrintString(str); Ypos += CHAR_HEIGHT; Xpos += CHAR_WIDTH;
	AVCXSetXY(Xpos, Ypos);
	AVCXSetRGBColor(0,0,0);
	AVCXPrintString(str); Ypos += CHAR_HEIGHT; Xpos += CHAR_WIDTH;
	AVCXSetXY(Xpos, Ypos);
	AVCXSetRGBColor(1,0,0);
	AVCXPrintString(str); Ypos += CHAR_HEIGHT; Xpos += CHAR_WIDTH;
	AVCXSetXY(Xpos, Ypos);
	AVCXSetRGBColor(0,1,0);
	AVCXPrintString(str); Ypos += CHAR_HEIGHT; Xpos -= CHAR_WIDTH;
	AVCXSetXY(Xpos, Ypos);
	AVCXSetRGBColor(0,0,1);
	AVCXPrintString(str); Ypos += CHAR_HEIGHT; Xpos -= CHAR_WIDTH;
	AVCXSetXY(Xpos, Ypos);
	AVCXSetRGBColor(1,1,0);
	AVCXPrintString(str); Ypos += CHAR_HEIGHT; Xpos -= CHAR_WIDTH;
	AVCXSetXY(Xpos, Ypos);
	AVCXSetRGBColor(1,0,0);
	AVCXPrintString(str); Ypos += CHAR_HEIGHT; Xpos -= CHAR_WIDTH;
	AVCXSetXY(Xpos, Ypos);
	AVCXSetRGBColor(0,0,0);
	AVCXPrintString(str); Ypos += CHAR_HEIGHT; Xpos -= CHAR_WIDTH;
	AVCXSetXY(Xpos, Ypos);
	return (12 * strlen(str));
}

int AVCXOnePage(char *str, int Xpos)
{
	AVCXClearScreen();
#if LCD_DMF50840
	return AVCXHalfPage(str, Xpos);
#elif LCD_EPSONP300013100
	return AVCXHalfPage(str, Xpos) + AVCXHalfPage(str, Xpos + AVCXPIXELS/2);
#endif
}

void TestPSetCodes(void)
{
#define deltaY 10
	int i;
	int y = 10;
	AVCXClearScreen();
	AVCXSetXY(0, LastLine - 8);								// pixel argument
	AVCXPrintString("Test PSet codes");
	AVCXSetXY(0, y);	//Starting Line
	while ((y + deltaY) <= AVCXLINES) {
		// draw a line
		for (i = 0; i < AVCXPIXELS / 2; ++i) {
			AVCXPSet(LCDPSET);
			AVCXPSet(LCDXPLUS);
		}
		for (i = AVCXPIXELS / 2; i < AVCXPIXELS; ++i) {
			AVCXPSet(LCDPSETXPLUS);
		}
		// move 10 lines down
		for (i = 0; i < deltaY; ++i) {
			AVCXPSet(LCDYPLUS);
			++y;
		}
	}
	AVCXPSet(LCDXMINUS);
	MySleep(1000);
	while (y >= deltaY) {
		// move 10 lines down
		for (i = 0; i < deltaY; ++i) {
			AVCXPSet(LCDYMINUS);
			--y;
		}
		// undraw a line
		for (i = AVCXPIXELS / 2; i > 0; --i) {
			AVCXPSet(LCDPUNSET);
			AVCXPSet(LCDXMINUS);
		}
		for (i = AVCXPIXELS; i > (AVCXPIXELS / 2); --i) {
			AVCXPSet(LCDPUNSETXMINUS);
		}
	}
}

void FillScreenWithText(void)
{
	char ch = ' ';
	int i,j;
	AVCXClearScreen();
	for (i = 0; i < (AVCXLINES / CHAR_HEIGHT); i++) {		//for each line
		AVCXSetXY(0, (i * CHAR_HEIGHT));			//start at beginning of line
		for (j = 0; j < (AVCXPIXELS / CHAR_WIDTH); j++) {
			AVCXPutChar(ch++);		//go through all 255 char values
		}
	}
}
//
// AVCX line drawing
//
void line(u16 x0, u16 y0, u16 x1, u16 y1)
{
	SerialPutc(&SerPrt, LCDDRAWLINE);
	SerialPutc(&SerPrt, (x0 & 0xff));
	SerialPutc(&SerPrt, (x0 >> 8));
	SerialPutc(&SerPrt, (y0 & 0xff));
	SerialPutc(&SerPrt, (y0 >> 8));
	SerialPutc(&SerPrt, (x1 & 0xff));
	SerialPutc(&SerPrt, (x1 >> 8));
	SerialPutc(&SerPrt, (y1 & 0xff));
	SerialPutc(&SerPrt, (y1 >> 8));
}
void DrawTriangle(unsigned int x1,unsigned int y1, unsigned int x2, unsigned int y2,unsigned int x3,unsigned int y3)
{
	if (x1 > (AVCXPIXELS - 1)) x1 = AVCXPIXELS - 1;
	if (x2 > (AVCXPIXELS - 1)) x2 = AVCXPIXELS - 1;
	if (x3 > (AVCXPIXELS - 1)) x3 = AVCXPIXELS - 1;
	if (y1 > (AVCXLINES - 1)) y1 = AVCXLINES - 1;
	if (y2 > (AVCXLINES - 1)) y2 = AVCXLINES - 1;
	if (y3 > (AVCXLINES - 1)) y3 = AVCXLINES - 1;

	line(x1, y1, x2, y2);
	line(x2, y2, x3, y3);
	line(x3, y3, x1, y1);
}

void AVCXLine(unsigned int x0, unsigned int y0, unsigned int x1, unsigned int y1, unsigned int color)
{
	AVCXSetBWColor(color);
	line (x0, y0, x1, y1);
}

void AVCXCircle(unsigned int x, unsigned int y, unsigned int radius, unsigned int color)
{
	AVCXSetBWColor(color);
	SerialPutc(&SerPrt, LCDDRAWCIRCLE);
	SerialPutc(&SerPrt, (x & 0xff));
	SerialPutc(&SerPrt, (x >> 8));
	SerialPutc(&SerPrt, (y & 0xff));
	SerialPutc(&SerPrt, (y >> 8));
	SerialPutc(&SerPrt, (radius & 0xff));
	SerialPutc(&SerPrt, (radius >> 8));
}

void AVCXSerialTest(int v)
{
	int i, cnt = 0;
	SerialPutc(&SerPrt, LCDSERIALTEST);
	SerialPutc(&SerPrt, (unsigned char)128);
	for (i = 0; i < 128; ++i) {
		SerialPutc(&SerPrt, (unsigned char)(v + i));
	}
}

#if 0
char RGB[] =
{
	0x07, 0x00,
	0x34, 0x00,
	0xc0, 0x01,
	0x07, 0x00,
	0x34, 0x00,
	0xc0, 0x01,
	0x07, 0x00,
	0x34, 0x00,
	0xc0, 0x01,
	0x07, 0x00,
	0x34, 0x00,
	0xc0, 0x01 };

//
// colorbars - 
BOOL SendTestBitMap(void)
{
	int i, j = 0;
	char b1 = 7, b2 = 0;
	int rgbIdx = 0;
	AVCXClearScreen();
	MySleep(1000);

	SerialPutc(&SerPrt, 0xc0); // Bitmap
	for (i = 0; i < 160; i++) {
		b1 = RGB[rgbIdx * 2];
		b2 = RGB[rgbIdx * 2 + 1];
		for (j = 0; j < AVCXLINES; j++) {
			SerialPutc(&SerPrt, b1); // Pixel
			SerialPutc(&SerPrt, b2); // Pixel
		}
		if ((i % 10) == 0) {
			++rgbIdx;
		}
	}
}
#endif

BOOL SendBitMap(BOOL Short, unsigned int HorPixelOffset, unsigned int VertOffset, char *pImageName, BOOL fInverted) {
//	BITMAPINFO	BM;
	BITMAPINFO	*pBM;	// = &BM;
	GLubyte * pImage;
	GLubyte * pOrigImage;
	WORD RedMask = 0xF800;
	WORD GreenMask = 0x07E0;
	WORD BlueMask = 0x001F;
	int RedShift = 10;
	int GreenShift = 5;
	int BlueShift = 0;
	int nPixels, nRows, i, j;
	BOOL first = TRUE;
	int clust = 0;
	int nBytes = 0;
	int size = 0;
	int rowSize;

	pImage = LoadDIBitmap(pImageName, &pBM);

	if (NULL == pImage) {
		DWORD err = GetLastError();
		return FALSE;
	}
	pOrigImage = pImage;

	if (pBM->bmiHeader.biBitCount != 1) { // Expect a monochrome image
		free(pOrigImage);
		free(pBM);
		return FALSE;
	}

	if ((nPixels = pBM->bmiHeader.biWidth) > AVCXPIXELS) {
		free(pOrigImage);
		free(pBM);
		return FALSE;
	}

	if ((nPixels + HorPixelOffset) >  AVCXPIXELS) {
		free(pOrigImage);
		free(pBM);
		return FALSE;
	}

	if ((nRows = pBM->bmiHeader.biHeight) > AVCXLINES) {
		int delta = (pBM->bmiHeader.biHeight - AVCXLINES);
		// skip initial 0.5 delta lines in image
		pImage = (GLubyte *)((DWORD)pImage + (delta/2) * (nPixels / 8)); //byte computation
		nRows -= delta;
	}

	if ((nRows + VertOffset) >  AVCXLINES) {
		int delta = (nRows + VertOffset) - AVCXLINES;
//		pImage = (GLubyte *)((DWORD)pImage + (delta/2) * (nPixels / 8)); //byte computation
		nRows -= delta;
	}

	if (pBM->bmiHeader.biCompression != BI_RGB) { // need straight bitmap
		free(pOrigImage);
		free(pBM);
		return FALSE;
	}

#if !GENBITMAP
//	printf("send bitmap: %d %d %d %d\n", HorPixelOffset/4, VertOffset, (HorPixelOffset/4 + (nPixels / 4) - 1), (VertOffset + (nRows - 1)));
	//
	// conversion to pixel argument for X: last pixel address doesn't match last nibble address
	//
	SerialPutc(&SerPrt, LCDBITMAP);							// Bitmap
	SerialPutc(&SerPrt, HorPixelOffset/4);					//x0		//TODO sb pixel address
	SerialPutc(&SerPrt, VertOffset);						//y0
	SerialPutc(&SerPrt, (HorPixelOffset/4 + (nPixels / 4) - 1)); //x1	//TODO sb pixel address
	SerialPutc(&SerPrt, (VertOffset + (nRows - 1)));		//y1
#endif

	for (i = 0; i < nRows; i++) {	// for each row
		rowSize = 0;
		for (j = 0; j < (nPixels/16); j++) { // send 16 bits at a time
			WORD pix = *pImage++;
			size += 2; rowSize += 2;
#if GENBITMAP
			if (first) {
//				printf("%-10s.db ","bitmap:");
				printf("%-10s.byte ","bitmap:");
				NumCount = 0;
				first = FALSE;
			} else {
//				if ((NumCount % 2) == 0)
			{
//					printf("          .db ");
					printf("          .byte ");
				}
			}

			b1 = ~(pix & 0xff);
			b2 = ~(pix >> 8) & 0xff;
			if (b1 == 0) printf("0x00,"); else
			printf("%#2.2x,", b1);
			if (b2 == 0) printf("0x00\n"); else
			printf("%#2.2x\n", b2);

#if 0
			NumCount++;
			if ((NumCount % 2) != 0) {
				printf(",");
			} else {
				printf("\n");
			}
#endif
#else
			if (fInverted) {
				// send 2 INVERTED nibbles per byte.
				SerialPutc(&SerPrt, ~(pix & 0xff));
				SerialPutc(&SerPrt, ~((pix >> 8) & 0xff));
			} else {
				// send 2 nibbles per byte.
				SerialPutc(&SerPrt, (pix & 0xff));
				SerialPutc(&SerPrt, ((pix >> 8) & 0xff));
			}
#endif
		}
		// each row is padded so that size is a multiple of 4
		if (rowSize & 0x03) {
			++pImage;
			size += 2;
		}
#if GENBITMAP
		printf("// End line %d\n", i);
#endif
	}
#if GENBITMAP
	printf("\n");
#endif

	if (size != pBM->bmiHeader.biSizeImage) {
//		printf("Size mismatch: header=%d count=%d\n", pBM->bmiHeader.biSizeImage, size);
	}
	if (nRows > AVCXLINES) {
		nRows -= AVCXLINES;
		// skip tail lines in image
		pImage = (GLubyte *)((DWORD)pImage + nRows * (nPixels / 8));	//byte computation
	} else if (nRows < AVCXLINES) {
		//TODO spit out dummy lines
		free(pOrigImage);
		free(pBM);
		return FALSE;
	} else if (nRows != AVCXLINES) {	//assert
		free(pOrigImage);
		free(pBM);
		return FALSE;
	}

	free(pOrigImage);
	free(pBM);
	return TRUE;
}			

BOOL SendFullBitMap(BOOL Short, char *pImageName, BOOL fInverted) {
	return SendBitMap(Short, 0, 0, pImageName, fInverted);
}

#define BITMAPNAME0 "Tree.BMP"
#define BITMAPNAME1 "Marvin.BMP"
#define BITMAPNAME2 "Rose.BMP"
#define BITMAPNAME3 "Church.BMP"
#define BITMAPNAME4 "Flower.BMP"
#define BITMAPNAME5 "Cartoon.BMP"
#define BITMAPNAME6 "bruceb.BMP"
#define BITMAPNAME7 "tulips.BMP"
#define BITMAPNAME8 "House.BMP"
#define NBITMAPS 9
#define BITMAPNAMEA "Planets.BMP"
#define BITMAPNAMEB "Planets2.BMP"
#define BITMAPNAMEC "Planets3.BMP"

char *ImageNames[NBITMAPS] = {
	BITMAPNAME0,
	BITMAPNAME1,
	BITMAPNAME2,
	BITMAPNAME3,
	BITMAPNAME4,
	BITMAPNAME5,
	BITMAPNAME6,
	BITMAPNAME7,
	BITMAPNAME8,
};

#if GENBITMAP
#define BITMAPNAME BITMAPNAME2
#endif

void ShowTriangles(unsigned int BaseX, unsigned int BaseY, unsigned int SleepVal)
{
	int i;
	if ((BaseX < 100) || (BaseX > 160) || (BaseY < 50) || (BaseY > 220)) {
		printf("ShowTriangles: Invalid arguments %d and %d\n", BaseX, BaseY);
	}
	AVCXSetBWColor(1);
	for (i = 0; i < 20; i++) {
		DrawTriangle(BaseX + (i * 5),BaseY - 50 + (i * 5), BaseX + 100 + (i * 3),BaseY + (i * 5), BaseX - 100 + (i * 5),BaseY + (i * 5));
	}
	MySleep(SleepVal);
	AVCXSetBWColor(0);
	for (i = 0; i < 20; i++) {
		DrawTriangle(BaseX + (i * 5),BaseY - 50 + (i * 5), BaseX + 100 + (i * 3),BaseY + (i * 5), BaseX - 100 + (i * 5),BaseY + (i * 5));
	}
	AVCXSetBWColor(1);
	MySleep(SleepVal);
}

void ShowGraphics(void)
{
	int i, y, lasti;
	AVCXClockOff();
	AVCXClearScreen();
	AVCXSetXY(0, LastLine - 8);								// pixel argument
	AVCXPrintString("Built-in Graphics");
	for (y = 7; y < AVCXLINES; y += 8) {					//Draw lines pattern
		AVCXLine(0, y, AVCXPIXELS - 1, y, 1);
	}

	for (i = 10; i < (AVCXLINES/2); i += 5) {				//Draw circles
		AVCXCircle(AVCXPIXELS/2, AVCXLINES/2, i, 1);
		MySleep(300);
	}
	lasti = i;
	for (i = lasti; i >= 10; i -= 5) {
		AVCXCircle(AVCXPIXELS/2, AVCXLINES/2, i, 0);		//Clear circles
		if (i == lasti) {									//Redraw text if necessary
			AVCXSetXY(0, LastLine - 8);						//Expects pixel argument
			AVCXPrintString("Built-in Graphics");
		}
		for (y = 7; y < AVCXLINES; y += 8) {				//Redraw lines is necessary
			if ((y >= (AVCXLINES/2 - i)) && (y <= (AVCXLINES/2 + i)))
				AVCXLine(0, y, AVCXPIXELS - 1, y, 1);
		}
	}
	for (y = 7; y < AVCXLINES; y += 8) {					//Clear lines pattern
		AVCXLine(0, y, AVCXPIXELS - 1, y, 0);
	}

	AVCXSetXY(0, LastLine - 8);								// pixel argument
	AVCXPrintString("Built-in Graphics");
	ShowTriangles(100, 50, 500);
	ShowTriangles(125, 75, 500);
	ShowTriangles(150, 100, 500);
	MySleep(500);
}
//
int mrandom(int max)
{
	int v = rand() % (max + 1);
	return v;
}
//
void FastBitMaps(int dwCnt)
{
	int i, j, lastRight, lastLeft;
	BOOL Invert;

	AVCXClearScreen();
	AVCXPrintString("Fast Bitmaps             ");
	MySleep(1000);
	lastLeft = lastRight = 0;
#if LCD_EPSONP300013100
	for (j = 0; j < dwCnt; ++j) {
		do {
			i = mrandom(NBITMAPS - 1);
		} while ((i == lastLeft) || (i == lastRight));
		lastLeft = i;
		Invert = (i == 2) ? TRUE : FALSE;
		SendFullBitMap(FALSE, ImageNames[i], Invert);
		do {
			i = mrandom(NBITMAPS - 1);
		} while ((i == lastLeft) || (i == lastRight));
		lastRight = i;
		Invert = (i == 2) ? TRUE : FALSE;
		SendBitMap(FALSE, AVCXPIXELS/2, 0, ImageNames[i], Invert);
	}
#else
	for (j = 0; j < dwCnt; ++j) {
		do {
			i = mrandom(NBITMAPS - 1);
		} while (i == lastLeft);
		lastLeft = i;
		Invert = (i == 2) ? FALSE : TRUE;
		SendFullBitMap(FALSE, ImageNames[i], Invert);
	}
#endif
	MySleep(1000);
}
//
// accept ONE argument digit 0..9 to indicate the serial port to use.
//
int main(int argc, char *argv[])
{
	int i, Xpos = 1;
	int	tmp, cnt = 0;
	char strbuf[256];
	int port = 3;
	char SerialPortStr[] = "\\\\.\\COM3";
	int baudrate = 115200;
	BOOL foundBaud = FALSE;
	BOOL fInvert, fInvert2;

	fInvert = FALSE;
	fInvert2 = TRUE;
#if LCD_DMF50840
	fInvert = TRUE;
	fInvert2 = FALSE;
#endif

#if GENBITMAP
//
// Dump generated AVCX bitmap data to stdout
// Use 8-bit colordepth
//
	SendBitMap(TRUE, 0, 0, BITMAPNAME);
#else	
	i = 1;
	while (argc-- > 1) {
		tmp = atoi(argv[i++]);
		if (tmp > 9)  {
			// assume this is the baudrate if not specified yet
			if (!foundBaud) {
				baudrate = tmp;
				foundBaud = TRUE;
			} else {
				printf("Invalid COM port number detected. Using COM0:\n");
				port = 0;
			}
		} else {
			port = tmp;
		}
	}
	SerialPortStr[7] = port + '0';
	printf("AVCXDemo V1.2. Using COM%c: at %d baud\n", port + '0', baudrate);
	SerPrt = SerialInit(SerialPortStr, baudrate);
	if (SerPrt == INVALID_HANDLE_VALUE) {
		printf("Unable to Open COM%c:\n", port + '0');
		return FALSE;
	}
#if 1
	AVCXSetRGBColor(1,0,0);
	AVCXClockOff();
	AVCXSetXY(0,0);
	AVCXPrintString("AVCXDemo V1.2");
	MySleep(1000);
	srand((unsigned int)GetTickCount());
	do {
		AVCXSetScreen(1, 0, 0, AVCXPIXELS - 1, AVCXLINES - 1);
		MySleep(1000);
		FastBitMaps(10);
		AVCXClearScreen();
		AVCXClockOn();
#if LCD_EPSONP300013100
		SendBitMap(FALSE, AVCXPIXELS/2, 0, BITMAPNAME8, FALSE);
#endif
		//
		// Display images in the right pane. Clock
		// size is AVCXLINES by AVCXLINES
		// Image size is 80 x 60
		//
		SendBitMap(FALSE, AVCXLINES + CLOCKOFFSET, 0, BITMAPNAMEC, TRUE);
		MySleep(500);
		SendBitMap(FALSE, AVCXLINES + CLOCKOFFSET, 60, BITMAPNAMEC, TRUE);
		MySleep(500);
		SendBitMap(FALSE, AVCXLINES + CLOCKOFFSET, 120, BITMAPNAMEC, TRUE);
		MySleep(500);
		SendBitMap(FALSE, AVCXLINES + CLOCKOFFSET, 180, BITMAPNAMEC, TRUE);
		MySleep(2000);
		AVCXClockOff();
		MySleep(1000);
		TestPSetCodes();
		AVCXDemo(1);
		MySleep(1000);
		AVCXDemo(2);
		MySleep(1000);
		AVCXDemo(3);
		MySleep(1000);
		AVCXDemo(4);
		FillScreenWithText();
		MySleep(2000);
		FastBitMaps(10);
		AVCXSetScreen(0, 0, 0, AVCXLINES + CLOCKOFFSET - 1, AVCXLINES - 1);
		AVCXSetScreen(1, AVCXLINES + CLOCKOFFSET, 0, AVCXPIXELS - 1, AVCXLINES - 1);
		AVCXSetBWColor(1);		// This sets the Background color back to BLACK
		AVCXClockOn();
		MySleep(2000);
		AVCXClockOff();
		MySleep(1000);
		AVCXSetScreen(1, 0, 0, AVCXPIXELS - 1, AVCXLINES - 1);
		AVCXSetBWColor(0);		// This sets the Background color back to WHITE
		AVCXClockOn();
		MySleep(2000);
		AVCXClockOff();
		MySleep(1000);
		ShowGraphics();
		cnt += AVCXOnePage("This is test1", Xpos);
		sprintf_s(strbuf, 256, "cnt = %d", cnt); //assume 0 termination
		AVCXPrintString(strbuf);
		MySleep(1000);
		SendFullBitMap(FALSE, BITMAPNAME0, fInvert);
#if LCD_EPSONP300013100
		SendBitMap(FALSE, AVCXPIXELS/2, 0, BITMAPNAME0, FALSE);
#endif
		MySleep(1500);
		SendFullBitMap(FALSE, BITMAPNAME1, fInvert);
		MySleep(1500);
		SendFullBitMap(FALSE, BITMAPNAME2, fInvert2);
		MySleep(1500);
		SendFullBitMap(FALSE, BITMAPNAME3, fInvert);
		MySleep(1500);
		SendFullBitMap(FALSE, BITMAPNAME4, fInvert);
		MySleep(1500);
		SendFullBitMap(FALSE, BITMAPNAME5, fInvert);
		MySleep(1500);
		SendFullBitMap(FALSE, BITMAPNAME6, fInvert);
		MySleep(1500);
		SendFullBitMap(FALSE, BITMAPNAME7, fInvert);
		MySleep(1500);
		SendFullBitMap(FALSE, BITMAPNAME8, fInvert);
		MySleep(1500);
		FillScreenWithText();
		MySleep(1500);
		AVCXClearScreen();
		AVCXClockOn();
		MySleep(2000);
		AVCXClockOff();
		MySleep(1000);
		ShowGraphics();
		//
		// Send partial images
		//
		AVCXClearScreen();
		SendBitMap(FALSE, 0, 0, BITMAPNAMEA, TRUE);
		MySleep(500);
#if LCD_DMF50840
		SendBitMap(FALSE, AVCXPIXELS/2, 0, BITMAPNAMEB, TRUE);
		MySleep(500);
		SendBitMap(FALSE, 0, AVCXLINES/2, BITMAPNAMEB, TRUE);
		MySleep(500);
		SendBitMap(FALSE, AVCXPIXELS/2, AVCXLINES/2, BITMAPNAMEA, TRUE);
#elif LCD_EPSONP300013100
		SendBitMap(FALSE, AVCXPIXELS/3, 0, BITMAPNAMEB, TRUE);
		MySleep(500);
		SendBitMap(FALSE, AVCXPIXELS/3*2, 0, BITMAPNAMEB, TRUE);
		MySleep(500);
		SendBitMap(FALSE, 0, AVCXLINES/2, BITMAPNAMEB, TRUE);
		MySleep(500);
		SendBitMap(FALSE, AVCXPIXELS/3, AVCXLINES/2, BITMAPNAMEA, TRUE);
		MySleep(500);
		SendBitMap(FALSE, AVCXPIXELS/3*2, AVCXLINES/2, BITMAPNAMEA, TRUE);
#endif
		MySleep(2000);

		cnt += AVCXOnePage("This is another test", Xpos);
		sprintf_s(strbuf, 256, "cnt = %d", cnt); //assume 0 termination
		AVCXPrintString(strbuf);
		MySleep(1000);
		AVCXClearScreen();
		AVCXClockOn();
		MySleep(2000);
		AVCXClockOff();
		MySleep(1000);
		SendFullBitMap(FALSE, BITMAPNAME2, fInvert);
		MySleep(1500);
		cnt += AVCXOnePage("How about another test", Xpos);
		sprintf_s(strbuf, 256, "cnt = %d", cnt); //assume 0 termination
		AVCXPrintString(strbuf);
		MySleep(1000);
		SendFullBitMap(FALSE, BITMAPNAME3, fInvert);
		MySleep(1500);
		cnt += AVCXOnePage("Let's do another test", Xpos);
		sprintf_s(strbuf, 256, "cnt = %d", cnt); //assume 0 termination
		AVCXPrintString(strbuf);
		MySleep(1000);
		SendFullBitMap(FALSE, BITMAPNAME4, fInvert);
		MySleep(1500);
		ShowGraphics();
		cnt += AVCXOnePage("Now onto test2", Xpos);
		sprintf_s(strbuf, 256, "cnt = %d", cnt); //assume 0 termination
		AVCXPrintString(strbuf);
		MySleep(1000);
		SendFullBitMap(FALSE, BITMAPNAME6, fInvert);
		MySleep(1500);
		cnt += AVCXOnePage("Another test2", Xpos);
		sprintf_s(strbuf, 256, "cnt = %d", cnt); //assume 0 termination
		AVCXPrintString(strbuf);
		MySleep(1000);
		SendFullBitMap(FALSE, BITMAPNAME1, fInvert);
		MySleep(1500);
		cnt += AVCXOnePage("More of test2", Xpos);
		sprintf_s(strbuf, 256, "cnt = %d", cnt); //assume 0 termination
		AVCXPrintString(strbuf);
		MySleep(1000);
		SendFullBitMap(FALSE, BITMAPNAME2, fInvert2);
		MySleep(1500);
		cnt += AVCXOnePage("Almost last test2", Xpos);
		sprintf_s(strbuf, 256, "cnt = %d", cnt); //assume 0 termination
		AVCXPrintString(strbuf);
		MySleep(1000);
		SendFullBitMap(FALSE, BITMAPNAME3, fInvert);
		MySleep(1500);
		cnt += AVCXOnePage("Almost almost last test2", Xpos);
		sprintf_s(strbuf, 256, "cnt = %d", cnt); //assume 0 termination
		AVCXPrintString(strbuf);
		MySleep(1000);
		SendFullBitMap(FALSE, BITMAPNAME4, fInvert);
		MySleep(1500);
		cnt += AVCXOnePage("Finally last test2", Xpos);
		sprintf_s(strbuf, 256, "cnt = %d", cnt); //assume 0 termination
		AVCXPrintString(strbuf);
		MySleep(1000);
		SendFullBitMap(FALSE, BITMAPNAME2, fInvert2);
		MySleep(1000);
		AVCXClearScreen();
		AVCXClockOn();
		MySleep(2000);
		AVCXClockOff();
		MySleep(1000);
		ShowGraphics();
		FastBitMaps(10);
		MySleep(2000);
		FillScreenWithText();
		MySleep(1500);
		Xpos += 20;
		if (Xpos > 100) Xpos = 0;
	} while (1);
#else
	printf("Start Serial Test\n");
	for (i = 0; i < 125; ++i)
	{
		printf("%d ", i);
		AVCXSerialTest(i);
	}
	// Last byte never seems to reach the target!
	SerialPutc(&SerPrt, (unsigned char)LCD_DUMMY);
	printf("Completed Serial Test\n");
#endif
#endif
return 0;

}

