#ifndef AVCX_INCLUDED
#define AVCX_INCLUDED
//
// AVCX Serial Protocol
//
// AVCX Graphics Protocol Changes V1.1
//
//	<SETX>. <SETY>, <SETX,PSET> & <SETY,PSET> now all take a 2 byte argument (pixel address)
//	<DRAW_LINE> command (0xA8) takes 4 two-byte arguments (pixel address): x0 (2), y0 (2) ,x1 (2) ,y1 (2)
//  <DRAW_CIRCLE> command (0xC8) takes 3 two-byte arguments (pixel address): x, y, radius
//	<DEMO> command (0xB8) takes a 1 byte argument (1,2,3 or 4)
//	<PICTURE> command now needs x0,y0 and x1,y1 nibble arguments before data bytes. Todo. sb pixel argument
// 		SetPicture takes x,y coordinates for upper left and lower right corners. Reverse order rows expected
//	<CLOCK_ON>, <CLOCK_OFF> commands.
//	<SETSCREEN> command with color and x,y coordinates (like <PICTURE>)
//
//	Todo.	add set time command
//			Add SetCluster/SetNibble inByte
//
// Jan 2011 changes: V1.2
//
//	. Added SERIALTEST, SERIALTEST_DATA
//	. Changed CLEAR_SCR, LIGHTON, LIGHTOFF, LCD_DUMMY, CLOCK_ON, CLOCK_OFF opcodes
//	. Added Clr Pixel to PSet Opcodes
//	. Changed SET Opcode
//	. TODO LCDDEMO to SingleByte opcodes
//
// Plan: set displays to default char mode (just show ascii chars on screen),
//		 then use an escape code to enable the AVCX protocol.
//		 Need LED or so to indicate current mode.
//		 Automatic timeout?
//		 Select Escape such that double enable is ok.
//
// AVCX Opcodes
//
// SingleByte: leftmost 2 bits are 01
//
//  bits		Hex
// 01 0XXXXX	4X-5X PSet (incl, coordinates Inc. & Dec.)
// 01 10XXXX	60-6F Reserved
// 01 110000	70
// 01 110001
// 01 110010
// 01 110011	73	CLEAR_SCR
// 01 110100
// 01 110101
// 01 110110
// 01 110111
// 01 111000	78	LIGHTON
// 01 111001	79	LIGHTOFF
// 01 111010
// 01 111011
// 01 111100
// 01 111101	7D	LCD_DUMMY
// 01 111110	7E	CLOCK_ON
// 01 111111	7F	CLOCK_OFF
//
//
// MultiByte (leftmost 5 bits, except SingleByte)
//
//	bits	Hex			 bits	Hex				 bits	Hex
// 00 000	00			10 000	80				11 000	C0	PICTURE
// 00 001	08			10 001	88	COLOR		11 001	C8	DRAW_CIRCLE
// 00 010	10			10 010	90	SET			11 010	D0	SERIALTEST
// 00 011	18			10 011	98	SET_SCREEN	11 011	D8	SERIALTEST_DATA
// 00 100	20			10 100	A0	PUT_CHAR	11 100	E0	PICTURE_DATA
// 00 101	28			10 101	A8	DRAW_LINE	11 101	E8
// 00 110	30			10 110	B0	PRINT_STRING 11 110	F0
// 00 111	38			10 111	B8	LCDDEMO		11 111	F8
//
// PSet: 5 bits
//
// bit 0: X
// bit 1: Y
// bit 2: Turn Pixel ON
// bit 3: 0 -> inc, 1 -> dec
// bit 4: Turn Pixel OFF
//
// if bits 2 or 5 are not set, no pixel change; just inc/dec of x/y
// if bits 0 or 1 are not set, no x/y action
// bits 0,1 both set: not used yet
// bits 2,4 both set: not used yet
//
// Set Opcode: 3 bits
//
// bit 0: 1 -> X, 0 -> Y
// bit 1: Turn Pixel OFF
// bit 2: Turn Pixel ON
//
// if bits 1 or 2 are not set, only X/Y set function used.
//
#define SINGLEBYTEMASK	0b11000000
#define SINGLEBYTECMD	0b01000000
#define MULTIBYTEMASK	0b11111000
#define PSETINCDECMASK	0b11100000
#define PSETINCDECCMD	0b01000000
#define PSETXYMASK		0b00000011
#define PSETPIXMASK		0b00010100
#define PSETX			_BV(0)
#define PSETY			_BV(1)
#define PSETPIXON		_BV(2)
#define PSETDEC			_BV(3)
#define PSETPIXOFF		_BV(4)
#define SETX			_BV(0)
#define SETPIXOFF		_BV(1)
#define SETPIXON		_BV(2)

#define	STANDBY		0x00
//Single-Byte commands
#define	CLEAR_SCR	0x73	//0b01110011
#define	LIGHTON		0x78	//0b01111000
#define	LIGHTOFF	0x79	//0b01111001
#define LCD_DUMMY	0x7D	//0b01111101
#define	CLOCK_ON	0x7E	//0b01111110
#define	CLOCK_OFF	0x7F	//0b01111111

//Multi-Byte commands
#define	COLOR		0x88
#define	SET			0x90		//SetX, SetY, SetX+Pset, SetY+Pset
#define	PUT_CHAR	0xA0
#define	DRAW_LINE	0xA8
#define	DRAW_CIRCLE	0xC8
#define	PRINT_STRING 0xB0
#define	PICTURE		0xC0
#define	SET_SCREEN	0x98
#define	PICTURE_DATA 0xE0
#define	LCDDEMO		0xB8		// Takes one argument to select demo

#if DEBUG
#define SERIALTEST	0xD0
#define SERIALTEST_DATA 0xD8
#endif

#endif
