//
// config.h
//
// wired as follows:
//  Serial port: PD0..PD1
//	MCU_TEST: PC0
//
//	I/O: PB0..PB3
//	YSCL: PD2
//	M: PD3
//	ENABLE: PD4
//  HSYNC: PD5
//	VSYNC: PD6
//	DOTCLK: PD7
//	ECL: PC1
//	DebugLED: PA0
//	DebugLED2: PA1
//
//
// PB0..PB3 connected to Data0..Data3 (nibble data)
//
#include "./Commoncfg.h"
#include "./avcx.h"

#define	SRAMDATAI	PINB
#define	SRAMDATAO	PORTB
#define	SRAMDATA_DDR DDRB
#define SRAMDATA_MASK (_BV(DB0_1) | _BV(DB1_2) | _BV(DB2_3) | _BV(DB3_4))
//
// PD2 connected to YSCL
//
#define	YSCL_DDR	DDRD
#define	LCD_YSCL	PORTD
//
// PD3 connected to M (AC) Line
//
#define	LCDM_DDR	DDRD
#define	LCD_M		PORTD
//
// PD4 connected to LCD Enable
//
#define	ENABLE_DDR	DDRD
#define	LCD_ENABLE	PORTD
//
// PD5 connected to LCD HSYNC (LP)
//
#define	LCD_HSYNC	PORTD
#define	HSYNC_DDR	DDRD
//
// PD6 connected to LCD VSYNC (SF)
//
#define	VSYNC_DDR	DDRD
#define	LCD_VSYNC	PORTD
//
// PD7 connected to software DOTCLOCK (XSCL)
//
#define	DOTCLK		PORTD
#define	DOTCLK_DDR	DDRD
//
// PC0 connected to MCU_TEST bit
//
#define MCU_TEST	 PORTD
#define MCU_TEST_DDR DDRC
//
// PC1 connected to LCD ECL
//
#define	LCD_ECL		PORTC
#define	ECL_DDR		DDRC
//
// PA0 connected to LED
//
#define	LEDI		PINA
#define	LEDO		PORTA
#define	LED_DDR		DDRA
//
// PA1 connected to LED2
//
#define	LED2I		PINA
#define	LED2O		PORTA
#define	LED2_DDR	DDRA

#ifndef cbi
	#define cbi(reg,bit)	reg &= ~(_BV(bit))
#endif
#ifndef sbi
	#define sbi(reg,bit)	reg |= (_BV(bit))
#endif

#define VERBOSE 1

typedef unsigned char  uint8;
typedef          char  int8;
typedef unsigned int   uint;
typedef unsigned long  ulong;

#define NOACTIVITY 600
#define RXSIZE 128
//
// LCD parameters
//
#define BITSPERPIXEL	1
#define DATABITS		4
#define BYTES_PER_LINE 160
#define NIBBLES_PER_LINE (2 * BYTES_PER_LINE)
#define LOG_NIBBLES_PER_LINE (NIBBLES_PER_LINE/2)
#define _PIXELS_PER_LINE 640
#define LOG_MAX_LINES 200
#define PIXS_PER_BYTE 8
//
#define nLines	LOG_MAX_LINES
#define lastLine (nLines - 1)
#define nHorPixels _PIXELS_PER_LINE
#define lastHorPixel (nHorPixels - 1)
#define	nColumnsPerLine ((nHorPixels * BITSPERPIXEL) / DATABITS)

#define FALSE	0
#define TRUE	1

#define LCDERROR0	0
#define LCDERROR1	1
#define LCDERROR2	2
#define LCDERROR3	3
#define LCDERROR4	4
#define LCDERROR5	5
#define LCDERROR6	6
#define LCDERROR7	7
#define LCDOVERFLOW 8


typedef uint8_t byte;
typedef uint8_t BOOL;
typedef unsigned long u32;
typedef			 int s16;

//
// GPIOR0 bits
//
#define		fRcvdChar	0
#define		fDebouncing	1
#define		fUpdSecond	2
#define		fUPS1		3
#define		fTimeoutS1	4
#define		fUPS2		5
#define		fTimeoutS2	6
#define		fFLASH		7
