/*
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software Foundation, 
  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <avr/io.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#include "./config.h"
#include "./ClkIsr.h"
#include "./util.h"

//
// These functions work only if TC1 is active
// Interrupts will be active but the Serial In Buffer
// will not be drained. Flow Control should prevent
// buffer overflow
//
void delayHalfSecond(void)
{
	Timer1Counter = 5;
	while (Timer1Counter) ;
}

void delay100msecs(void)
{
	Timer1Counter = 1;
	while (Timer1Counter) ;
}

void delay300msecs(void)
{
	Timer1Counter = 3;
	while (Timer1Counter) ;
}

void delayOneSecond(void)
{
	Timer1Counter = 10;
	while (Timer1Counter) ;
}

//
// Start an LED flash.
//
void FlashLed(uint8 duration, uint8 cnt)
{
	if (!LedDuration & ! LedCount) {
		SavedLedDuration = LedDuration = duration;		// timer1 is running at this time so first interval uncertain
		LedCount = (cnt + 1) & 0xfe;					// Must be even to have LED off at the end
		LED_ON;
	}
}

void BlinkFastNoInts(void)
{
#if USE_LED
	for (int i = 0; i < 10; ++i) {
		LED_ON;
		_delay_ms(100);
		LED_OFF;
		_delay_ms(100);
	}
#endif
}
//
// Run another timer, which is then used to blink an LED
// concurrently. Requires interrupts to be running
//
void BlinkFast(void)
{
#if USE_LED
	for (int i = 0; i < 10; ++i) {
		LED_ON;
		delay100msecs();
		LED_OFF;
		delay100msecs();
	}
#endif
}

void ShortBlinkFast(void)
{
#if USE_LED
	for (int i = 0; i < 3; ++i) {
		LED_ON;
		delay100msecs();
		LED_OFF;
		delay100msecs();
	}
#endif
}

void BlinkSlow(void)
{
#if USE_LED
	for (int i = 0; i < 10; ++i) {
		LED_ON;
		delay300msecs();
		LED_OFF;
		delay300msecs();
	}
#endif
}

const char AssertStr[] PROGMEM = "ASSERT:";
const char LineStr[] PROGMEM = " line ";

void foo(void);

void assertFailed(const char* str, int line)
{
    UARTPrintfProgStr(AssertStr);
	if (NULL != str) {
	    UARTPrintfStr((char *)str);
	}
    UARTPrintfProgStr(LineStr);
    UARTPrintfint(line);
    FlashLed(5, 200);
	sei();
	foo();
}

void PrintfEndOfLine(void)
{
	UARTSendByte('\r', FALSE);
	UARTSendByte('\n', FALSE);
}

void PrintfU4(uint8 Data)
{
    /* Send 4-bit hex value */
    uint8 Character = Data&0x0f;
    if (Character>9)
    {
        Character+='A'-10;
    }
    else
    {
        Character+='0';
    }
    UARTSendByte(Character, FALSE);
}

void Printfuint8(uint8 Data)
{
    /* Send 8-bit hex value */
    PrintfU4(Data>>4);
    PrintfU4(Data);
}

void Printfu16(uint Data)
{
    /* Send 16-bit hex value */
    Printfuint8(Data>>8);
    Printfuint8(Data);
}

void Printfuint(uint Data)
{
    /* Send 16-bit hex value */
    Printfuint8(Data>>8);
    Printfuint8(Data);
}

